@@ -26,7 +26,13 @@ static XtmSettings *settings;
 static GtkWidget *window;
 static GtkStatusIcon *status_icon;
 static XtmTaskManager *task_manager;
-static gboolean timeout = 0;
+static guint timeout = 0;
+static gboolean start_hidden = FALSE;
+
+static GOptionEntry main_entries[] = {
+	{ "start-hidden", 0, G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &start_hidden, "Don't open a task manager window", NULL },
+	{ NULL, 0, 0, 0, NULL, NULL, NULL }
+};
 
 static void
 status_icon_activated (void)
@@ -93,25 +99,25 @@ init_timeout (void)
 
 	xtm_task_manager_get_system_info (task_manager, &num_processes, &cpu, &memory_used, &memory_total, &swap_used, &swap_total);
 
-	memory_percent = (memory_total != 0) ? memory_used * 100 / (gdouble)memory_total : 0;
-	swap_percent = (swap_total != 0) ? swap_used * 100 / (gdouble)swap_total : 0;
+	memory_percent = (memory_total != 0) ? ((memory_used * 100.0f) / (float)memory_total) : 0.0f;
+	swap_percent = (swap_total != 0) ? ((swap_used * 100.0f) / (float)swap_total) : 0.0f;
 
 	g_object_get (settings, "show-memory-in-xbytes", &show_memory_in_xbytes, NULL);
 	if (show_memory_in_xbytes) {
 	  used = g_format_size_full(memory_used, G_FORMAT_SIZE_IEC_UNITS);
 	  total = g_format_size_full(memory_total, G_FORMAT_SIZE_IEC_UNITS);
-	  g_snprintf (memory_info, 64,"%s / %s", used, total);
+	  g_snprintf (memory_info, sizeof(memory_info), "%s / %s", used, total);
 	  g_free(used);
 	  g_free(total);
 
 	  used = g_format_size_full(swap_used, G_FORMAT_SIZE_IEC_UNITS);
 	  total = g_format_size_full(swap_total, G_FORMAT_SIZE_IEC_UNITS);
-	  g_snprintf (swap_info, 64,"%s / %s", used, total);
+	  g_snprintf (swap_info, sizeof(swap_info), "%s / %s", used, total);
 	  g_free(used);
 	  g_free(total);
 	} else {
-	  g_snprintf (memory_info, 64, "%.0f%%", memory_percent);
-	  g_snprintf (swap_info, 64, "%.0f%%", swap_percent);
+	  g_snprintf (memory_info, sizeof(memory_info), "%.0f%%", memory_percent);
+	  g_snprintf (swap_info, sizeof(swap_info), "%.0f%%", swap_percent);
 	}
 
 	xtm_process_window_set_system_info (XTM_PROCESS_WINDOW (window), num_processes, cpu, memory_percent, memory_info, swap_percent, swap_info);
@@ -122,7 +128,7 @@ init_timeout (void)
 	if (gtk_status_icon_get_visible (status_icon))
 	{
 #if GTK_CHECK_VERSION (2,16,0)
-		g_snprintf (tooltip, 1024,
+		g_snprintf (tooltip, sizeof(tooltip),
 				_("<b>Processes:</b> %u\n"
 				"<b>CPU:</b> %.0f%%\n"
 				"<b>Memory:</b> %s\n"
@@ -130,7 +136,7 @@ init_timeout (void)
 				num_processes, cpu, memory_info, swap_info);
 		gtk_status_icon_set_tooltip_markup (GTK_STATUS_ICON (status_icon), tooltip);
 #else
-		g_snprintf (tooltip, 1024,
+		g_snprintf (tooltip, sizeof(tooltip),
 				_("Processes: %u\n"
 				"CPU: %.0f%%\n"
 				"Memory: %s\n"
@@ -175,6 +181,7 @@ int main (int argc, char *argv[])
 #if GLIB_CHECK_VERSION(2, 28, 0)
 	GApplication *app;
 	GError *error = NULL;
+	GOptionContext *opt_context;
 #endif
 #ifdef ENABLE_NLS
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
@@ -186,6 +193,15 @@ int main (int argc, char *argv[])
 	g_set_application_name (_("Task Manager"));
 
 #if GLIB_CHECK_VERSION(2, 28, 0)
+	opt_context = g_option_context_new ("");
+	g_option_context_add_main_entries (opt_context, main_entries, NULL);
+	g_option_context_add_group (opt_context, gtk_get_option_group (TRUE));
+	if (!g_option_context_parse (opt_context, &argc, &argv, &error))
+	{
+		g_print ("Unable to parse arguments: %s\n", error->message);
+		return 1;
+	}
+
 	app = g_application_new ("xfce.taskmanager", 0);
 	g_application_register (G_APPLICATION (app), NULL, &error);
 	if (error != NULL)
@@ -197,7 +213,8 @@ int main (int argc, char *argv[])
 
 	if (g_application_get_is_remote (G_APPLICATION (app)))
 	{
-		g_application_activate (G_APPLICATION (app));
+		if (!start_hidden)
+			g_application_activate (G_APPLICATION (app));
 		g_object_unref (app);
 		return 0;
 	}
@@ -211,13 +228,15 @@ int main (int argc, char *argv[])
 	g_signal_connect (status_icon, "popup-menu", G_CALLBACK (status_icon_popup_menu), NULL);
 
 	window = xtm_process_window_new ();
-	gtk_widget_show (window);
+
+	if (!start_hidden)
+		gtk_widget_show (window);
+
 #if GLIB_CHECK_VERSION(2, 28, 0)
 	g_signal_connect_swapped (app, "activate", G_CALLBACK (xtm_process_window_show), window);
 #endif
 
 	task_manager = xtm_task_manager_new (xtm_process_window_get_model (XTM_PROCESS_WINDOW (window)));
-	g_message ("Running as %s on %s", xtm_task_manager_get_username (task_manager), xtm_task_manager_get_hostname (task_manager));
 
 	init_timeout ();
 	g_signal_connect (settings, "notify::refresh-rate", G_CALLBACK (refresh_rate_changed), NULL);
@@ -229,7 +248,10 @@ int main (int argc, char *argv[])
 	g_signal_connect (window, "destroy", G_CALLBACK (destroy_window), NULL);
 	g_signal_connect (window, "delete-event", G_CALLBACK (delete_window), NULL);
 
-	gtk_main ();
+	if (gtk_widget_get_visible (window) || gtk_status_icon_get_visible (status_icon))
+		gtk_main ();
+	else
+		g_warning ("Nothing to do: activate hiding to the notification area when using --start-hidden");
 
 	if (timeout > 0)
 		g_source_remove (timeout);
