@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2010 Mike Massonnet, <mmassonnet@xfce.org>
+ * Copyright (c) 2018 Rozhuk Ivan <rozhuk.im@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,6 +12,7 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <pwd.h>
@@ -53,9 +55,6 @@ struct _XtmTaskManager
 #endif
 	GtkTreeModel *		model;
 	GArray *		tasks;
-	guint			owner_uid;
-	gchar *			owner_uid_name;
-	gchar *			hostname;
 	gushort			cpu_count;
 	gfloat			cpu_user;
 	gfloat			cpu_system;
@@ -71,18 +70,11 @@ G_DEFINE_TYPE (XtmTaskManager, xtm_task_manager, G_TYPE_OBJECT)
 static void	xtm_task_manager_finalize			(GObject *object);
 
 static void	setting_changed					(GObject *object, GParamSpec *pspec, XtmTaskManager *manager);
-#ifdef HAVE_WNCK
-static void	model_add_task					(GtkTreeModel *model, Task *task, App *app, glong timestamp);
-static void	model_update_tree_iter				(GtkTreeModel *model, GtkTreeIter *iter, Task *task, App *app);
-static void	model_update_task				(GtkTreeModel *model, Task *task, App *app);
-#else
-static void	model_add_task					(GtkTreeModel *model, Task *task, glong timestamp);
-static void	model_update_tree_iter				(GtkTreeModel *model, GtkTreeIter *iter, Task *task);
-static void	model_update_task				(GtkTreeModel *model, Task *task);
-#endif
-static void	model_mark_tree_iter_as_removed			(GtkTreeModel *model, GtkTreeIter *iter);
+static void	model_add_task					(XtmTaskManager *manager, Task *task, glong timestamp);
+static void	model_update_tree_iter				(XtmTaskManager *manager, GtkTreeIter *iter, glong timestamp, gboolean update_cmd_line, Task *task);
+static void	model_mark_tree_iter_as_removed			(GtkTreeModel *model, GtkTreeIter *iter, glong timestamp);
 static void	model_remove_tree_iter				(GtkTreeModel *model, GtkTreeIter *iter);
-static void	model_find_tree_iter_for_pid			(GtkTreeModel *model, guint pid, GtkTreeIter *iter);
+static gboolean	task_list_find_for_pid				(GArray *task_list, GPid pid, Task **task, guint *idx);
 static glong	__current_timestamp				(void);
 
 
@@ -102,8 +94,6 @@ xtm_task_manager_init (XtmTaskManager *manager)
 	manager->app_manager = xtm_app_manager_new ();
 #endif
 	manager->tasks = g_array_new (FALSE, FALSE, sizeof (Task));
-	get_owner_uid (&(manager->owner_uid), &(manager->owner_uid_name));
-	manager->hostname = get_hostname ();
 
 	/* Listen to settings changes and force an update on the whole model */
 	settings = xtm_settings_get_default ();
@@ -118,8 +108,6 @@ xtm_task_manager_finalize (GObject *object)
 {
 	XtmTaskManager *manager = XTM_TASK_MANAGER (object);
 	g_array_free (manager->tasks, TRUE);
-	g_free (manager->owner_uid_name);
-	g_free (manager->hostname);
 #ifdef HAVE_WNCK
 	g_object_unref (manager->app_manager);
 #endif
@@ -127,7 +115,7 @@ xtm_task_manager_finalize (GObject *object)
 }
 
 static void
-setting_changed (GObject *object, GParamSpec *pspec, XtmTaskManager *manager)
+setting_changed (GObject *object, GParamSpec *pspec __unused, XtmTaskManager *manager __unused)
 {
 	g_object_get (object, "more-precision", &more_precision, NULL);
 	g_object_get (object, "full-command-line", &full_cmdline, NULL);
@@ -137,16 +125,21 @@ setting_changed (GObject *object, GParamSpec *pspec, XtmTaskManager *manager)
 static gchar *
 pretty_cmdline (gchar *cmdline, gchar *comm)
 {
-	gchar *text = g_strchomp (g_strdelimit (g_strdup (cmdline), "\n\r", ' '));
-	if (!full_cmdline && g_utf8_strlen (text, -1) > 3)
+	/* Use the printable range of 0x20-0x7E */
+	const gchar *valid_chars = " !\"#$%&'()*+,-./0123456789:;<=>?@"
+				   "ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`"
+				   "abcdefghijklmnopqrstuvwxyz{|}~";
+	gchar *text = g_strstrip (g_strcanon (g_strdup (cmdline), valid_chars, ' '));
+	gsize text_size = (gsize)strlen (text);
+	if (!full_cmdline && text_size > 3)
 	{
 		/* Shorten full path to commands and wine applications */
 		if (text[0] == '/' || (g_ascii_isupper (text[0]) && text[1] == ':' && text[2] == '\\'))
 		{
-			gchar *p = g_strstr_len (text, -1, comm);
+			gchar *p = g_strstr_len (text, (gssize)text_size, comm);
 			if (p != NULL)
 			{
-				g_strlcpy (text, p, g_utf8_strlen (text, -1));
+				g_strlcpy (text, p, text_size);
 			}
 		}
 	}
@@ -162,53 +155,33 @@ _xtm_task_manager_set_model (XtmTaskManager *manager, GtkTreeModel *model)
 }
 
 static void
-#ifdef HAVE_WNCK
-model_add_task (GtkTreeModel *model, Task *task, App *app, glong timestamp)
-#else
-model_add_task (GtkTreeModel *model, Task *task, glong timestamp)
-#endif
+model_add_task (XtmTaskManager *manager, Task *task, glong timestamp)
 {
 	GtkTreeIter iter;
-	gchar *cmdline;
-
-#ifdef HAVE_WNCK
-	if (app != NULL && full_cmdline == FALSE)
-		cmdline = g_strdup (app->name);
-	else
-		cmdline = pretty_cmdline (task->cmdline, task->name);
-#else
-	cmdline = pretty_cmdline (task->cmdline, task->name);
-#endif
+	GtkTreeModel *model = manager->model;
+	gchar *uid_name = get_uid_name (task->uid);
 
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-		XTM_PTV_COLUMN_COMMAND, cmdline,
 		XTM_PTV_COLUMN_PID, task->pid,
 		XTM_PTV_COLUMN_STATE, task->state,
 		XTM_PTV_COLUMN_UID, task->uid,
-		XTM_PTV_COLUMN_UID_STR, task->uid_name,
-		XTM_PTV_COLUMN_BACKGROUND, NULL,
-		XTM_PTV_COLUMN_FOREGROUND, NULL,
+		XTM_PTV_COLUMN_UID_STR, uid_name,
 		XTM_PTV_COLUMN_TIMESTAMP, timestamp,
 		-1);
-#ifdef HAVE_WNCK
-	model_update_tree_iter (model, &iter, task, app);
-#else
-	model_update_tree_iter (model, &iter, task);
-#endif
-
-	g_free (cmdline);
+	g_free(uid_name);
+	model_update_tree_iter (manager, &iter, timestamp, TRUE, task);
 }
 
 static void
-model_mark_tree_iter_as_removed (GtkTreeModel *model, GtkTreeIter *iter)
+model_mark_tree_iter_as_removed (GtkTreeModel *model, GtkTreeIter *iter, glong timestamp)
 {
 	gtk_list_store_set (GTK_LIST_STORE (model), iter,
 		XTM_PTV_COLUMN_CPU, 0.0,
 		XTM_PTV_COLUMN_CPU_STR, "-",
 		XTM_PTV_COLUMN_BACKGROUND, "#e57373",
 		XTM_PTV_COLUMN_FOREGROUND, "#000000",
-		XTM_PTV_COLUMN_TIMESTAMP, __current_timestamp (),
+		XTM_PTV_COLUMN_TIMESTAMP, timestamp,
 		-1);
 }
 
@@ -219,26 +192,24 @@ model_remove_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
 }
 
 static void
-#ifdef HAVE_WNCK
-model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task, App *app)
-#else
-model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
-#endif
+model_update_tree_iter (XtmTaskManager *manager, GtkTreeIter *iter, glong timestamp, gboolean update_cmd_line, Task *task)
 {
+	GtkTreeModel *model = manager->model;
 	gchar *vsz, *rss, cpu[16];
 	gchar value[14];
 	glong old_timestamp;
 	gchar *old_state;
 	gchar *background, *foreground;
 #ifdef HAVE_WNCK
+	App *app = xtm_app_manager_get_app_from_pid (manager->app_manager, task->pid);
 	GdkPixbuf *icon;
 #endif
 
 	vsz = g_format_size_full (task->vsz, G_FORMAT_SIZE_IEC_UNITS);
 	rss = g_format_size_full (task->rss, G_FORMAT_SIZE_IEC_UNITS);
 
-	g_snprintf (value, 14, (more_precision) ? "%.2f" : "%.0f", task->cpu_user + task->cpu_system);
-	g_snprintf (cpu, 16, _("%s%%"), value);
+	g_snprintf (value, sizeof(value), (more_precision) ? "%.2f" : "%.0f", (task->cpu_user + task->cpu_system));
+	g_snprintf (cpu, sizeof(cpu), _("%s%%"), value);
 
 	/* Retrieve values for tweaking background/foreground color and updating content as needed */
 	gtk_tree_model_get (model, iter, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, XTM_PTV_COLUMN_STATE, &old_state,
@@ -258,9 +229,9 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 		gtk_list_store_set (GTK_LIST_STORE (model), iter, XTM_PTV_COLUMN_COMMAND, cmdline, -1);
 		g_free (cmdline);
 	}
-	else if (model_update_forced)
+	else if (update_cmd_line)
 #else
-	if (model_update_forced)
+	if (update_cmd_line)
 #endif
 	{
 		gchar *cmdline = pretty_cmdline (task->cmdline, task->name);
@@ -273,10 +244,10 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 		/* Set yellow color for changing state */
 		background = g_strdup ("#fff176");
 		foreground = g_strdup ("#000000");
-		old_timestamp = __current_timestamp () - TIMESTAMP_DELTA + 3;
+		old_timestamp = timestamp - TIMESTAMP_DELTA + 3;
 	}
 
-	if (__current_timestamp () - old_timestamp <= TIMESTAMP_DELTA)
+	if (timestamp != 0 && (timestamp - old_timestamp) <= TIMESTAMP_DELTA)
 	{
 		/* Set green color for started task */
 		background = (background == NULL) ? g_strdup ("#aed581") : background;
@@ -296,7 +267,7 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 		XTM_PTV_COLUMN_VSZ_STR, vsz,
 		XTM_PTV_COLUMN_RSS, task->rss,
 		XTM_PTV_COLUMN_RSS_STR, rss,
-		XTM_PTV_COLUMN_CPU, task->cpu_user + task->cpu_system,
+		XTM_PTV_COLUMN_CPU, (task->cpu_user + task->cpu_system),
 		XTM_PTV_COLUMN_CPU_STR, cpu,
 		XTM_PTV_COLUMN_PRIORITY, task->prio,
 		XTM_PTV_COLUMN_BACKGROUND, background,
@@ -311,36 +282,29 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 	g_free (rss);
 }
 
-static void
-#ifdef HAVE_WNCK
-model_update_task (GtkTreeModel *model, Task *task, App *app)
-#else
-model_update_task (GtkTreeModel *model, Task *task)
-#endif
+static gboolean
+task_list_find_for_pid (GArray *task_list, GPid pid, Task **task, guint *idx)
 {
-	GtkTreeIter iter;
-	model_find_tree_iter_for_pid (model, task->pid, &iter);
-#ifdef HAVE_WNCK
-	model_update_tree_iter (model, &iter, task, app);
-#else
-	model_update_tree_iter (model, &iter, task);
-#endif
-}
+	Task *task_tmp, tkey;
+	tkey.pid = pid;
+	task_tmp = bsearch(&tkey, task_list->data, task_list->len, sizeof(Task), task_pid_compare_fn);
 
-static void
-model_find_tree_iter_for_pid (GtkTreeModel *model, guint pid, GtkTreeIter *iter)
-{
-	gboolean valid;
-	guint pid_iter;
-
-	valid = gtk_tree_model_get_iter_first (model, iter);
-	while (valid)
+	if (NULL != task)
 	{
-		gtk_tree_model_get (model, iter, XTM_PTV_COLUMN_PID, &pid_iter, -1);
-		if (pid == pid_iter)
-			break;
-		valid = gtk_tree_model_iter_next (model, iter);
+		(*task) = task_tmp;
 	}
+	if (NULL != idx)
+	{
+		if (NULL != task_tmp)
+		{
+			(*idx) = (guint)(((size_t)task_tmp - (size_t)task_list->data) / sizeof(Task));
+		}
+		else
+		{
+			(*idx) = (~((guint)0));
+		}
+	}
+	return ((NULL != task_tmp));
 }
 
 static glong
@@ -361,20 +325,6 @@ xtm_task_manager_new (GtkTreeModel *model)
 	return manager;
 }
 
-const gchar *
-xtm_task_manager_get_username (XtmTaskManager *manager)
-{
-	g_return_val_if_fail (XTM_IS_TASK_MANAGER (manager), NULL);
-	return manager->owner_uid_name;
-}
-
-const gchar *
-xtm_task_manager_get_hostname (XtmTaskManager *manager)
-{
-	g_return_val_if_fail (XTM_IS_TASK_MANAGER (manager), NULL);
-	return manager->hostname;
-}
-
 void
 xtm_task_manager_get_system_info (XtmTaskManager *manager, guint *num_processes, gfloat *cpu,
 				  guint64 *memory_used, guint64 *memory_total,
@@ -419,187 +369,102 @@ xtm_task_manager_get_task_list (XtmTaskManager *manager)
 void
 xtm_task_manager_update_model (XtmTaskManager *manager)
 {
-	static GArray *removed_tasks = NULL;
 	GArray *array;
 	guint i;
+	GtkTreeIter iter;
+	gboolean valid, need_update, update_cmd_line;
+	glong timestamp;
 
 	g_return_if_fail (XTM_IS_TASK_MANAGER (manager));
 
-	if (removed_tasks == NULL)
-		removed_tasks = g_array_new (FALSE, FALSE, sizeof (GtkTreeIter));
-
-	/* Retrieve initial task list and return */
-	if (manager->tasks->len == 0)
-	{
-		get_task_list (manager->tasks);
-		for (i = 0; i < manager->tasks->len; i++)
-		{
-			Task *task = &g_array_index (manager->tasks, Task, i);
-#ifdef HAVE_WNCK
-			App *app = xtm_app_manager_get_app_from_pid (manager->app_manager, task->pid);
-			model_add_task (manager->model, task, app, 0);
-#else
-			model_add_task (manager->model, task, 0);
-#endif
-#if DEBUG
-			g_print ("%5d %5s %15s %.50s\n", task->pid, task->uid_name, task->name, task->cmdline);
-#endif
-		}
-		return;
-	}
+	/* First time fast refresh. */
+	timestamp = ((manager->tasks->len == 0) ? 0 : __current_timestamp ());
 
 	/* Retrieve new task list */
 	array = g_array_new (FALSE, FALSE, sizeof (Task));
 	get_task_list (array);
 
-	/* Remove terminated tasks */
-	for (i = 0; i < removed_tasks->len; i++)
+	/* Remove terminated tasks, mark to remove, update existing. */
+	valid = gtk_tree_model_get_iter_first (manager->model, &iter);
+	while (valid)
 	{
-		GtkTreeIter *iter = &g_array_index (removed_tasks, GtkTreeIter, i);
+		GPid pid;
+		gchar *cpu_str;
 		glong old_timestamp;
-		gtk_tree_model_get (manager->model, iter, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, -1);
-		if (__current_timestamp () - old_timestamp > TIMESTAMP_DELTA)
-		{
-#if DEBUG
-			gint pid;
-			gtk_tree_model_get (manager->model, iter, XTM_PTV_COLUMN_PID, &pid, -1);
-			g_debug ("Remove old task %d", pid);
-#endif
-			model_remove_tree_iter (manager->model, iter);
-			g_array_remove_index (removed_tasks, i);
-		}
-	}
+		gboolean found;
+		Task *task, *task_new;
+		GtkTreeIter cur_iter = iter;
 
-	for (i = 0; i < manager->tasks->len; i++)
-	{
-		guint j;
-		Task *tasktmp;
-		Task *task = &g_array_index (manager->tasks, Task, i);
-		gboolean found = FALSE;
-
-		for (j = 0; j < array->len; j++)
+		valid = gtk_tree_model_iter_next (manager->model, &iter);
+		gtk_tree_model_get (manager->model, &cur_iter, XTM_PTV_COLUMN_CPU_STR, &cpu_str, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, XTM_PTV_COLUMN_PID, &pid, -1);
+		found = (g_strcmp0 (cpu_str, "-") == 0);
+		g_free (cpu_str);
+		if (found && (timestamp - old_timestamp) > TIMESTAMP_DELTA)
 		{
-			tasktmp = &g_array_index (array, Task, j);
-			if (task->pid != tasktmp->pid)
-				continue;
-			found = TRUE;
-			break;
+			G_DEBUG_FMT ("Remove old task %d", pid);
+			model_remove_tree_iter (manager->model, &cur_iter);
+			continue;
 		}
 
-		if (found == FALSE)
+		/* Looking for new terminated tasks and check updates for existing. */
+		found = task_list_find_for_pid (manager->tasks, pid, &task, &i);
+		if (FALSE == found)
 		{
-			GtkTreeIter iter;
-#if DEBUG
-			g_debug ("Task %d '%s' terminated, marking as removed", task->pid, task->name);
-#endif
-			model_find_tree_iter_for_pid (manager->model, task->pid, &iter);
-			model_mark_tree_iter_as_removed (manager->model, &iter);
-			g_array_append_val (removed_tasks, iter);
+			G_DEBUG_FMT ("Cached task not found for pid: %d", pid);
+			continue;
+		}
+		found = task_list_find_for_pid (array, pid, &task_new, NULL);
+		if (FALSE == found) /* Mark as removed. */
+		{
+			G_DEBUG_FMT ("Task %d '%s' terminated, marking as removed", pid, task->name);
+			model_mark_tree_iter_as_removed (manager->model, &cur_iter, timestamp);
 			g_array_remove_index (manager->tasks, i);
+			continue;
+		}
+
+		/* Task alive, check for update. */
+		need_update = FALSE;
+		update_cmd_line = FALSE;
+
+		/* Update the model (with the rest) only if needed, this keeps the CPU cool */
+		if (model_update_forced || 0 != memcmp(task, task_new, sizeof(Task)))
+		{
+			need_update = TRUE;
+			/* Update command name if needed (can happen) */
+			update_cmd_line = (model_update_forced || g_strcmp0 (task->cmdline, task_new->cmdline));
+			memcpy(task, task_new, sizeof(Task));
+		}
+		else /* Update color if needed */
+		{
+			gchar *color;
+
+			gtk_tree_model_get (manager->model, &cur_iter, XTM_PTV_COLUMN_BACKGROUND, &color, -1);
+			if (color != NULL && (timestamp - old_timestamp) > TIMESTAMP_DELTA)
+			{
+				need_update = TRUE;
+				G_DEBUG_FMT ("Remove color from running PID %d", pid);
+			}
+			g_free (color);
+		}
+
+		if (need_update)
+		{
+			model_update_tree_iter (manager, &cur_iter, timestamp, update_cmd_line, task);
 		}
 	}
 
 	/* Append started tasks and update existing ones */
 	for (i = 0; i < array->len; i++)
 	{
-		guint j;
 		Task *tasktmp = &g_array_index (array, Task, i);
-		gboolean found = FALSE;
 
-		for (j = 0; j < manager->tasks->len; j++)
-		{
-			Task *task = &g_array_index (manager->tasks, Task, j);
-#ifdef HAVE_WNCK
-			App *app;
-#endif
-			gboolean updated = FALSE;
-
-			if (task->pid != tasktmp->pid)
-				continue;
-
-#ifdef HAVE_WNCK
-			app = xtm_app_manager_get_app_from_pid (manager->app_manager, task->pid);
-#endif
-			found = TRUE;
-
-			/* Update the model (with the rest) only if needed, this keeps the CPU cool */
-			if (model_update_forced
-				|| task->ppid != tasktmp->ppid
-				|| g_strcmp0 (task->state, tasktmp->state)
-				|| task->cpu_user != tasktmp->cpu_user
-				|| task->cpu_system != tasktmp->cpu_system
-				|| task->rss != tasktmp->rss
-				|| task->vsz != tasktmp->vsz
-				|| task->prio != tasktmp->prio)
-			{
-				updated = TRUE;
-				task->ppid = tasktmp->ppid;
-				g_strlcpy (task->state, tasktmp->state, sizeof (task->state));
-				task->cpu_user = tasktmp->cpu_user;
-				task->cpu_system = tasktmp->cpu_system;
-				task->rss = tasktmp->rss;
-				task->vsz = tasktmp->vsz;
-				task->prio = tasktmp->prio;
-#ifdef HAVE_WNCK
-				model_update_task (manager->model, tasktmp, app);
-#else
-				model_update_task (manager->model, tasktmp);
-#endif
-			}
-
-			/* Update command name if needed (can happen) */
-			if (!model_update_forced && g_strcmp0 (task->cmdline, tasktmp->cmdline))
-			{
-				GtkTreeIter iter;
-				gchar *cmdline;
-
-				cmdline = pretty_cmdline (tasktmp->cmdline, tasktmp->name);
-				model_find_tree_iter_for_pid (manager->model, task->pid, &iter);
-				gtk_list_store_set (GTK_LIST_STORE (manager->model), &iter, XTM_PTV_COLUMN_COMMAND, cmdline, -1);
-				g_free (cmdline);
-			}
-
-			/* Update color if needed */
-			if (updated == FALSE)
-			{
-				GtkTreeIter iter;
-				gchar *color;
-				glong old_timestamp;
-
-				model_find_tree_iter_for_pid (manager->model, task->pid, &iter);
-				gtk_tree_model_get (manager->model, &iter, XTM_PTV_COLUMN_BACKGROUND, &color, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, -1);
-
-				if (color != NULL && __current_timestamp () - old_timestamp > TIMESTAMP_DELTA)
-				{
-#if DEBUG
-					g_debug ("Remove color from running PID %d", task->pid);
-#endif
-#ifdef HAVE_WNCK
-					model_update_task (manager->model, tasktmp, app);
-#else
-					model_update_task (manager->model, tasktmp);
-#endif
-				}
-
-				g_free (color);
-			}
-
-			break;
-		}
-
-		if (found == FALSE)
-		{
-#ifdef HAVE_WNCK
-			App *app = xtm_app_manager_get_app_from_pid (manager->app_manager, tasktmp->pid);
-			model_add_task (manager->model, tasktmp, app, __current_timestamp ());
-#else
-			model_add_task (manager->model, tasktmp, __current_timestamp ());
-#endif
-			g_array_append_val (manager->tasks, *tasktmp);
-#if DEBUG
-			g_debug ("Add new task %d %s", tasktmp->pid, tasktmp->name);
-#endif
-		}
+		if (task_list_find_for_pid (manager->tasks, tasktmp->pid, NULL, NULL))
+			continue;
+		model_add_task (manager, tasktmp, timestamp);
+		/* XXX: add bininsert() here. */
+		g_array_append_val (manager->tasks, *tasktmp);
+		g_array_sort (manager->tasks, task_pid_compare_fn);
+		G_DEBUG_FMT ("Add new task %d %s", tasktmp->pid, tasktmp->name);
 	}
 
 	g_array_free (array, TRUE);
@@ -609,37 +474,21 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 }
 
 
-
-void
-get_owner_uid (guint *owner_uid, gchar **owner_uid_name)
-{
-	uid_t uid;
-	struct passwd *pw;
-	gchar *username = NULL;
-
-	uid = getuid ();
-	pw = getpwuid (uid);
-
-	username = g_strdup ((pw != NULL) ? pw->pw_name : "nobody");
-
-	*owner_uid = (guint) uid;
-	*owner_uid_name = username;
-}
-
 gchar *
-get_hostname (void)
+get_uid_name (guint uid)
 {
-#ifndef HOST_NAME_MAX
-#define HOST_NAME_MAX 255
-#endif
-	char hostname[HOST_NAME_MAX];
-	if (gethostname (hostname, HOST_NAME_MAX))
-		return g_strdup ("(unknown)");
-	return g_strdup_printf ("%s", hostname);
+	int error;
+	struct passwd *pw = NULL, pwd_buf;
+	char buf[4096];
+
+	memset(buf, 0, sizeof(buf));
+	error = getpwuid_r(uid, &pwd_buf, buf, sizeof(buf), &pw);
+
+	return (g_strdup ((0 == error && pw != NULL) ? pw->pw_name : "nobody"));
 }
 
 gboolean
-send_signal_to_pid (guint pid, gint xtm_signal)
+send_signal_to_pid (GPid pid, gint xtm_signal)
 {
 	gint sig;
 	gint res;
@@ -665,7 +514,7 @@ send_signal_to_pid (guint pid, gint xtm_signal)
 }
 
 gboolean
-set_priority_to_pid (guint pid, gint priority)
+set_priority_to_pid (GPid pid, gint priority)
 {
 	gint prio;
 	gint res;
@@ -693,3 +542,8 @@ set_priority_to_pid (guint pid, gint priority)
 	return (res == 0) ? TRUE : FALSE;
 }
 
+gint
+task_pid_compare_fn(gconstpointer a, gconstpointer b)
+{
+	return (((const Task*)a)->pid - ((const Task*)b)->pid);
+}
